/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.client.internal.PlansClient");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

(function() {

/*
 * Aliases
 */
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

/*
 * Constants
 */
var _SERVICE_URI = "com.ibm.team.dashboard.viewlets.service.internal.plans.IPlansService";
var _APT_SERVICE_URI = "com.ibm.team.apt.internal.service.rest.IPlanRestService";

com.ibm.team.dashboard.viewlets.web.client.internal.PlansClient.getDevelopmentLines =
/* DevelopmentLineDTO[] */ function(/* string */ itemId, /* string */ itemScope, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
	var req = new ServiceRequest(_SERVICE_URI, "getDevelopmentLines", {itemId: itemId, itemScope: itemScope});
	if (tracker) {
		tracker.register(req, srh);
	}
	TeamServerClient.invokeService(req, srh);
}

com.ibm.team.dashboard.viewlets.web.client.internal.PlansClient.getCurrentIteration =
/* IterationDTO */ function(/* string */ lineItemId, /* string */ teamItemId, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
	if (!this._iterationsMap)
		this._iterationsMap = new Object();
	var serviceObject = {
		self: this,
		success: function (dto) {
			this.self._iterationsMap[dto.itemId] = dto;
			srh.doCallback(dto);
		},
		failure: function (error) {
			srh.doErrorCallback(error);
		}
	}
	var srh2 = new ServiceResponseHandler(serviceObject, "success", "failure");
	var req = new ServiceRequest(_SERVICE_URI, "getCurrentIteration", {lineItemId: lineItemId, teamItemId: teamItemId});
	if (tracker) {
		tracker.register(req, srh2);
	}
	TeamServerClient.invokeService(req, srh2);
}

com.ibm.team.dashboard.viewlets.web.client.internal.PlansClient.getIteration =
/* IterationDTO */ function(/* string */ itemId, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
	if (!this._iterationsMap)
		this._iterationsMap = new Object();
	if (this._iterationsMap[itemId]) {
		if (this._iterationsMap[itemId] == "fetching") {
			if (!this._iterationsCallbackQueue)
				this._iterationsCallbackQueue = new Object();
			if (!this._iterationsCallbackQueue[itemId])
				this._iterationsCallbackQueue[itemId] = [];
			this._iterationsCallbackQueue[itemId].push(srh);
		} else
			srh.doCallback(this._iterationsMap[itemId]);
		return;
	} else
		this._iterationsMap[itemId] = "fetching";
	var serviceObject = {
		self: this,
		success: function (dto) {
			this.self._iterationsMap[itemId] = dto;
			if (this.self._iterationsCallbackQueue && this.self._iterationsCallbackQueue[itemId]) {
				for (var i = 0; i < this.self._iterationsCallbackQueue[itemId].length; i++)
					this.self._iterationsCallbackQueue[itemId][i].doCallback(dto);
			}
			srh.doCallback(dto);
		},
		failure: function (error) {
			srh.doErrorCallback(error);
		}
	}
	var srh2 = new ServiceResponseHandler(serviceObject, "success", "failure");
	var req = new ServiceRequest(_SERVICE_URI, "getIteration", {itemId: itemId});
	if (tracker) {
		tracker.register(req, srh2);
	}
	TeamServerClient.invokeService(req, srh2);
}

com.ibm.team.dashboard.viewlets.web.client.internal.PlansClient.getDevelopmentLineById =
/* DevelopmentLineDTO[] */ function(/* string */ id, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
	var req = new ServiceRequest(_SERVICE_URI, "getDevelopmentLineById", {id: id});
	if (tracker) {
		tracker.register(req, srh);
	}
	TeamServerClient.invokeService(req, srh);
}

// expose the APT services with our own client so we can change project area
com.ibm.team.dashboard.viewlets.web.client.internal.PlansClient.searchIterationPlans =
/* SearchIterationPlansResultDTO */ function(params, srh, /* ServiceTracker */ tracker) {
	if (!params.projectAreaId)
		dojo.mixin(params, { projectAreaId: getCurrentProjectArea().itemId });
	var req = new ServiceRequest(_APT_SERVICE_URI, "getIterationPlanSearchResults", params);
	if (tracker) {
		tracker.register(req, srh);
	}
	TeamServerClient.invokeService(req, srh);
}

})();