/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.client.util"); 

dojo.require("dojo.date");
dojo.require("dojox.date.posix");
dojo.require("dojo.i18n");
dojo.require("dojox.string.Builder");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNonNegative		= net.jazz.ajax.toolbox.lang.checkNonNegative;

var HEX_DIGITS			= ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "E","F"];
var WORKITEM_ID_REGEX		= /#?([0-9]+)/;

// constants for this.breakupLongWords
if (dojo.isIE && dojo.isIE < 7) {
	var MIN_BREAKUP_LEN			= 32;
	var MAX_LEN					= 40;
} else {
	var MIN_BREAKUP_LEN			= 40;
	var MAX_LEN					= 52;
}

var DELIMITERS				= [".", "-", "_", "/", "?", "#"];

com.ibm.team.workitem.web.client.util= {
	messages: dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages"),
	DIALOG_DURATION: 100, // Standard fade duration for work item dialogs (short and sweet)

	_basicPrefix: "#action=com.ibm.team.workitem.",
	
	// DO NOT DELETE without first doing a search across the workitem.web
	// plug-in (being used by WorkItemOverview.js as of 8 Dec 2006)
	_viewWorkItemURIPrefix: "#action=com.ibm.team.workitem.viewWorkItem&id=",
	
	getEditQueryURI: function(queryId) {
		checkNotNull(queryId, "queryId");
		return this._basicPrefix + "editQuery&id=" + queryId;
	},
	getWelcomePageURI: function(msg) {
		var suffix = "";
		if(msg) {
			suffix = "&error=" + escape(msg);
		} 
		return this._basicPrefix + "viewWelcome" + suffix;
	},
	
	getNewWorkItemURI: function() {
		return this._basicPrefix + "newWorkItem";
	},
	
	getNewQueryURI: function() {
		return this.reuniquifyURL(this._basicPrefix + "newQuery");
	},
	getRunDefaultQueryURI: function(includeTimestamp) {
		if(includeTimestamp) {
			return this.reuniquifyURL(this._basicPrefix + "runDefaultQuery");			
		} 
		return this._basicPrefix + "runDefaultQuery";
	},
	isChildOf: function(startNode, parentNode) {
			var testNode = startNode;
			if(parentNode == testNode) return true;

			// Check to see if the popup is a parent of the target
			var priorParent = null;
			while(testNode != null && testNode != priorParent && testNode != document.body && testNode != document) {
				priorParent = testNode;
				if(testNode==parentNode) {
					return true;
				}
				testNode=testNode.parentNode;				
			}
			return false;
	},
	/*
	 * Answer true if the node is a child of a dijit popup 
	 */	
	isChildOfPopup: function(startNode) {

			var testNode = startNode;
			var priorParent = null;
			while(testNode != null && testNode != priorParent && testNode != document.body && testNode != document) {
				priorParent = testNode;
				if(testNode.dijitPopupParent) {
					return true;
				}
				testNode=testNode.parentNode;				
			}
			return false;
	},	
	getRunSavedQueryURI: function(queryId, index, orderBy, direction, itemsPerPage, isUnique, preserveFilter, filterAttribute, filterValue) {
		checkNotNull(queryId, "queryId");
		return this._basicPrefix + "runSavedQuery&id=" + queryId +
				(index ? "&index=" + index : "") +
				(orderBy ? "&orderBy=" + orderBy : "") +  
				(direction ? "&direction=" + direction : "") + 
				(itemsPerPage ? "&itemsPerPage=" + itemsPerPage : "") +
				(preserveFilter ? "&preserveFilter=true" : "") +
				(filterAttribute ? "&filterAttribute=" + filterAttribute : "") +
				(filterValue ? "&filterValue=" + encodeURIComponent(filterValue) : "") +
				this._getUniqueSuffix(isUnique);
	},
	
	getRunNewQueryURI: function(/* QueryDTO */ query, /* int */ index, /* String */ orderBy, /* String */ direction, /* String */ itemsPerPage, preserveFilter) {
		checkNotNull(query, "query");
		var serialized= this.serializeQuery(query);
		return this._basicPrefix + "runNewQuery&q=" + serialized +
				(index ? "&index=" + index : "") +
				(orderBy ? "&orderBy=" + orderBy : "") + 
				(preserveFilter ? "&preserveFilter=true" : "") +
				(direction ? "&direction=" + direction : "") +
				(itemsPerPage ? "&itemsPerPage=" + itemsPerPage : "");
	},

	getSimilarityQueryURI: function( /* itemId, content, summary */ parms) {
		if(parms.itemId) { // Existing work item search
			return this._basicPrefix + "similarityQuery&itemId=" + parms.itemId
		}
		var summary, content;
		if(parms.summary && parms.summary.length>0) {
			summary = "&summary=" + encodeURIComponent(dojo.trim(parms.summary)) 
		} else {
			summary= "";
		}

		if(parms.content && parms.content.length>0) {
			content = "&content=" + encodeURIComponent(dojo.trim(parms.content)) 
		} else {
			content= "";
		}
		
		return this._basicPrefix + "similarityQuery" + summary + content;
	},
	getViewWorkItemURI: function(id, query, tab) {
		checkNotNull(id, "id");
		return this._getViewSingleWorkItemURI(id, tab);
	},
	getURLParam: function(url, param) {
	  var exp = "[.*&]"+param+"=([^&#]*)";
  	  var regex = new RegExp(exp);
  	  var result = regex.exec(url);
      if( !result || result.length < 2) return null;
      return result[1];
	},
	_getViewSingleWorkItemURI: function(id, tab) {
		checkNotNull(id, "id");
	
		var idNum= parseInt(id, 10);
		checkNonNegative(idNum, "idNum");
		
		return  this._basicPrefix + "viewWorkItem&id=" + id + (tab ? "&tab=" + tab : "");
	},
	getQueryURI: function(query, index, orderBy, direction, itemsPerPage, preserveFilter) {
		checkNotNull(query, "query");
		
  		if(!query.isNew && query.itemId)  {
  			return this.getRunSavedQueryURI(query.itemId, index, orderBy, direction, itemsPerPage, false, preserveFilter);
  		}
	  		
  		return this.getRunNewQueryURI(query, index, orderBy, direction, itemsPerPage, preserveFilter);
	},
	/*
	 * Return the fragment, which is the part after the first #
	 */
	getFragment: function(url) {
		return new dojo._Url(url).fragment; /* TODO: Remove reference to internal dojo method */
	},
	getSearchURI: function(unencodedStr, openOnly, isUnique) {
		checkNotNull(unencodedStr, "unencodedStr");
		var encodedFragment= encodeURIComponent(dojo.trim(unencodedStr));
		
		if (openOnly) {
			return this._basicPrefix + "search&q=" + encodedFragment + "&o=" + openOnly  + this._getUniqueSuffix(isUnique);
		} else {
			return this._basicPrefix + "search&q=" + encodedFragment + this._getUniqueSuffix(isUnique);
		}
	},
	reuniquifyURL: function(/* string */ url, removePreserveFilter) {
		if(removePreserveFilter) {
			url = url.replace(/&preserveFilter=true/, "");
		}
		return url.replace(/&tq=([0-9]+)/, "") + "&tq=" + this.getUniqueId();
	}, 
	reuniquifyURL2: function(/* string */ url) {
		return url.replace(/&ts=([0-9]+)/, "") + "&ts=" + this.getUniqueId();
	},
	_getUniqueSuffix: function(isUnique) {
		if(!isUnique) return "";
		return "&tq=" + this.getUniqueId();
	},
//	Return a unique id for use in urls
	getUniqueId: function() {
		return Date.parse(new  Date().toString());
//		return this._uniqueId++;
	},
	/*
	 * Returns the default query page size for the Work Item vertical.  Should
	 * not be accessed by code outside the workitem.web bundle.
	 */
	_getDefaultQueryPageSize: function() {
		return 50;
	},
	/*
	 * Returns the maximum query page size for the Work Item vertical.  Should
	 * not be accessed by code outside the workitem.web bundle.
	 */
	_getMaximumQueryPageSize: function() {
		return 500;
	},
	_getHashRequestParams: function(href) {
		if(!href) {
			href = document.location.href;
		}
		var reqParms= [];
		var frag= this.getFragment(href);
		if (frag) {
			var nvps= frag.split("&");
			for (var i=0; i<nvps.length; i++) {
				var nvp= nvps[i].split("=");
				if (nvp.length === 2) {
					reqParms.push({name: nvp[0], value: nvp[1]});
				}
			}
		}
		return reqParms;
	},
	
	/* boolean */ isWorkItemId: function(/* String */ id) {
		return WORKITEM_ID_REGEX.test(id);
	},
	
	/* int */ getWorkItemId: function(/* String */ id) {
		var match= WORKITEM_ID_REGEX.exec(id);
		return parseInt(match[1]);
	},
	
	/* String */ serializeQuery: function(/* QueryDTO */ query) {
		var json= dojo.toJson(query);
		return this._toHex(json);
	},
	
	/* QueryDTO */ deserializeQuery: function(/* String */ hexString) {
		var someCode= this._fromHex(hexString);
		return this._doEval(someCode);
	},
	
	_doEval: function(someCode) {
		try {
			return eval("(" + someCode + ")");
		}catch(e) {
			console.debug(e);
			return someCode;
		}
	},
	
	/* String */ _toHex: function(/* String */ str) {
		var builder= new dojox.string.Builder();
		for (var i= 0; i < str.length; i++) {
			var cc= str.charCodeAt(i);
			var mod= cc % 16;
			var x1= HEX_DIGITS[mod];
			var x2= HEX_DIGITS[(cc-mod) / 16];
			builder.append(x2, x1);
		}
		return builder.toString();
	},
	
	/* String */ _fromHex: function(/* String */ str) {
		var builder= new dojox.string.Builder();
		for (var i= 0; i < str.length; i+= 2) {
			var hexChar= str.charAt(i) + str.charAt(i+1);
			var decChar= parseInt(hexChar, 16);
			builder.append(String.fromCharCode(decChar));
		}
		return builder.toString();
	},
		
	_breakupAllLongWords: function(paragraph, minLength) {
		if(!minLength) {
			minLength =MIN_BREAKUP_LEN;
		}
		// break the overall paragraph into strings based on spaces
		var words= paragraph.split(" ");
		var needsBreakingUp= false;
		// two pass algorithm so that we don't have to do a lot of string
		// copying for the 99% case
		for (var i=0; i<words.length; i++) {
			if (words[i].length > minLength) {
				needsBreakingUp= true;
				break;
			}
		}
		if (!needsBreakingUp) {
			return paragraph;
		}
		// if we're still running we have some long words, so now we'll do
		// some string manipulation
		var retStr= "";
		for (var i=0; i<words.length; i++) {
			// if this is a short word, add it to the return string and
			// loop again, else break up the word
			// If the word is an href, then don't split it
			if ( (words[i].indexOf("href=") != -1) || words[i].length < minLength ) {
				retStr += words[i];
			} else {
				retStr += this._breakupLongWord(words[i], minLength);
			}
			retStr += " ";
		}
		return retStr;
	},
	
	_breakupLongWord: function(word, minLength) {
		if(!minLength) {
			minLength =MIN_BREAKUP_LEN;
		}
		var remainingWord= word;
		var brokenupWord= "";
		while (remainingWord.length > minLength) {
			var splitPoint= minLength + 1;
			for (var i=0; i<DELIMITERS.length; i++) {
				var delim= DELIMITERS[i];
				var end= (remainingWord.length > MAX_LEN ? MAX_LEN : remainingWord.length) + 1;
				var candidateSplitSubstr= remainingWord.substring(minLength + 1, end);
				var delimIndex= candidateSplitSubstr.indexOf(delim);
				if (delimIndex != -1) {
					splitPoint=  minLength + delimIndex + 2;	
					break;
				}
			}
			brokenupWord += remainingWord.substring(0, splitPoint) + " ";
			remainingWord = remainingWord.substring(splitPoint, remainingWord.length);
		}
		brokenupWord += remainingWord;
		return brokenupWord;
	},
	isDebugMode: function() {
			return location.href.indexOf("debug=true") != -1;
	},
	/*
	 * Return a date given a formatted string
	 */ 
	fromISOString: function(timestamp) {
		//Example:  2003-10-06T06:46:00.000-0400
		// dojo.date.stamp.fromISOString wants:
		//          2005-06-30T08:05:00-07:00
		if(timestamp.length == 28) {
			timestamp = timestamp.substring(0,19) + "-" + timestamp.substring(24, 26) + ":" + timestamp.substring(26,28);
		}
		return dojo.date.stamp.fromISOString(timestamp);
		
	},	
	getAllTimestamps: function(modified) {
		var resultArgs = {};
		resultArgs.sortDate = this.fromISOString(modified);
		resultArgs.printTimestamp = com.ibm.team.workitem.web.client.util.getModifiedLabel(modified, /* isIso */ true);
		resultArgs.formattedTimestamp = dojo.date.locale.format(resultArgs.sortDate, {formatLength: "medium"});
		resultArgs.modifiedTimestamp = modified;
		return resultArgs;
	},
	getDateLabel: function(date) {
		if(!date || date=="Unassigned") {
			return this.messages.labelNone;
		}
		return dojo.date.locale.format(date, {formatLength: "medium"});
	},
	setServerTimeDelta: function(time) {
		var serverTime= this.fromISOString(time);
		this.serverTimeDelta = new Date().getTime() - serverTime.getTime();
	},
	getModifiedLabel: function(modified, isIso) {
			isIso = isIso == null ? false : isIso;

			if(isIso) {
				var modDate= this.fromISOString(modified);
			} else {
				var modDate = modified;
			}
			
			var now = new Date(new Date().getTime() - this.serverTimeDelta);
			var deltaMinutes= dojo.date.difference(modDate, now, "minute" );
			
			// If clock is wrong or different time zone
			if(deltaMinutes < 0) {
				return dojo.date.locale.format(modDate, {formatLength: 'medium' })
//				return  dojox.date.posix.strftime(modDate, "%c"); // preferred date and time in current locale
			}
			
			// if less than an hour ago, display in minutes
			if (deltaMinutes < 60) {
				if (deltaMinutes === 0) {
					deltaMinutes= 1;
				}
				if(deltaMinutes === 1) {
					return this.messages.labelDateMinuteAgo;
				}
				return dojo.string.substitute(this.messages.labelDateMinutesAgo, [deltaMinutes ]);
			}
			
			var yearOffset= dojo.date.difference(modDate, now, "year");
			var totalDayOffset= dojo.date.difference(modDate, now, "day");
			
			// if same day or less than 12 hours ago, display in hours
			if (totalDayOffset === 0 || deltaMinutes < 720) {
				var hours= Math.round(deltaMinutes / 60);
				if(hours === 1) {
					return this.messages.labelDateHourAgo;
				}
				return dojo.string.substitute(this.messages.labelDateHoursAgo, [hours]);
			}
	
			// if previous calendar date, display "Yesterday"
			if (totalDayOffset === 1) {
				return this.messages.labelDateYesterday;
			}
			
			// if less than 7 days, display the number of days
			if (totalDayOffset < 7) {
				return dojo.string.substitute(this.messages.labelDateDaysAgo, [totalDayOffset ]);
			}
			
			// if greater then 7 days, but less than 14, display "Last Week"
			if (totalDayOffset < 14) {
				return this.messages.labelDateLastWeek;
			}
	
			if (dojo.i18n.normalizeLocale() == "en-us") {
				// if same year, return (e.g.) Friday, Jan 24
				// if not same year, return (e.g.) Aug 7, 1977
				if (yearOffset === 0) {
					return dojox.date.posix.strftime(modDate, "%A, %b %e");
				}
				else {
					return dojox.date.posix.strftime(modDate, "%b %e, %Y");
				}
			} else {
				return dojo.date.locale.format(modDate, {selector:'date', formatLength: 'medium'});
//				return dojox.date.posix.strftime(modDate, "%x"); // preferred date in the current locale
			}
		},

	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @return boolean
 	 */	
	isStringDefined: function(text) {
		if (text == null) {
			return false;
		} else if (typeof(text) != 'string') {
			return false;
		} else if (text.length == 0) {
			return false;
		}
		return true;
	},
	getLeftNavString: function(text) {
		if(!text) return this.messages.labelUnnamed;
		if(text.length > 500) {
			text = text.substring(0,500) + "...";
		}
		text = this._breakupAllLongWords(text, 20);
		return this.plainToHTML(text);
	},
	plainToHTML: function(plain) {
		var span= document.createElement("span");
		var text= document.createTextNode(plain);
		span.appendChild(text);
		return span.innerHTML;
	},
		
	buildMap: function(array, keyName, valueName) {
		
		if (!array)
			return {};
		
		if (!keyName)
			keyName= "key";
		
		if (!valueName)
			valueName= "value";
			
		var map= new Object();
		for (var i in array) {
			var element= array[i];
			map[element[keyName]]= element[valueName];
		}
		return map;
	}
};

})();
