/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.JazzFeedViewlet");

dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.FeedClient");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.FeedViewlet");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dojo.i18n");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "JazzFeedMessages");

(function() {

/*
 * Aliases
 */
var FeedClient = com.ibm.team.dashboard.viewlets.web.client.internal.FeedClient;
var FeedViewlet = com.ibm.team.dashboard.viewlets.web.ui.internal.FeedViewlet;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;

var messages;

/*
 * A feed viewlet specific to Jazz feeds.
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.JazzFeedViewlet",
	FeedViewlet, {
	
	constructor: function() {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "JazzFeedMessages");
		this.setColumnCount(2);
	},
	
	onInit: function() {
		this._updateTitle();
	},
	
	fetch: function (/* ServiceResponseHandler */ srh, /* number */ startIndex, /* number */ count, /* boolean */ entries, /* boolean */ details) {
		delete this._errorMsgNode;
		// handle Auto(none) scope when no team areas
		if (!this.getScopeItem()) {
			this._errorMsgNode = this._createNoScopeMsg();
			this._entryCount = 0;
			this._updateSecondaryTitle();
			srh.doCallback();
			return;
		}
		var serviceObj = {
			_srh: srh,
			_viewlet: this,
			_entries: entries,
			_details: details,
			success: function(/* FeedResponseDTO */ response) {
				this._viewlet.handleFetch(response, this._entries, this._details, this._srh);
			},
			error: function(/* Error */ error) {
				this._srh.doErrorCallback(error);
			}
		};
		var srh2 = new ServiceResponseHandler(serviceObj, "success", "error");
		FeedClient.getJazzFeed(this.getFilters(), this.getScopeItem().itemId, this.getScope(), this.getShowActiveBuilds(), this.getScopeSubteams(), null, this.getReferenceItemId(), startIndex, count, entries, details, !this.getFeedInfo(), srh2, this.getServiceTracker());
	},
	
	getFilters: function() {
		var result = new Array();
		if (this.getPreference("build"))
			result.push("build");
		if (this.getPreference("dashboard"))
			result.push("dashboard");
		if (this.getPreference("news"))
			result.push("news");
		if (this.getPreference("reports"))
			result.push("reports");
		if (this.getPreference("changes"))
			result.push("changes");
		if (this.getPreference("stream"))
			result.push("stream");
		if (this.getPreference("process"))
			result.push("process");
		if (this.getPreference("workItems"))
			result.push("workItems");
		return result;
	},
	
	getNoItemsMessage: function() {
		return messages.noEntries;
	},
	
	getShowActiveBuilds: function() {
		return true;
	},

	fetchTooltip: function(/* EntryDTO */ entry, /* ServiceResponseHandler */ srh) {
		FeedClient.getJazzFeed(this.getFilters(), this.getScopeItem().itemId, this.getScope(), this.getShowActiveBuilds(), this.getScopeSubteams(), entry.items[0].linkURL, null, null, null, true, true, false, srh, this.getServiceTracker());
	},
	
	settingsChanged: function(oldSettings, newSettings) {
		var scopeChanged = false;
		var updateTitle = false;
		if (newSettings.scope != oldSettings.scope || newSettings.scopeItemId != oldSettings.scopeItemId) {
			updateTitle = true;
			scopeChanged = true;
		}
		if (updateTitle || newSettings.userTitle != oldSettings.userTitle)
			this._updateTitle();
		if (scopeChanged || newSettings.scope != DashboardConstants.SCOPE_CONTRIBUTOR && newSettings.scopeSubteams != oldSettings.scopeSubteams
			|| oldSettings.preferences.build != newSettings.preferences.build
			|| oldSettings.preferences.dashboard != newSettings.preferences.dashboard
			|| oldSettings.preferences.news != newSettings.preferences.news
			|| oldSettings.preferences.reports != newSettings.preferences.reports
			|| oldSettings.preferences.changes != newSettings.preferences.changes
			|| oldSettings.preferences.stream != newSettings.preferences.stream
			|| oldSettings.preferences.process != newSettings.preferences.process
			|| oldSettings.preferences.workItems != newSettings.preferences.workItems) {
			this.reset(true);
			this.getSite().refresh();
		}
		else {
			this.update();
			this.updateTotalPageFooter();
		}
	},
	
	_createNoScopeMsg: function() {
		var msgSpan = document.createElement("span");
		msgSpan.innerHTML = messages.notScoped;
		if (this.getSite().isDashboardEditing()) {
			var editSpan = document.createElement("span");
			var links = I18nUtil.addMessage(editSpan, messages.notScopedEdit);
			this.connect(links[0], "onclick", this._handleToggleSettings);
			msgSpan.appendChild(editSpan);
		}
		return msgSpan;
	},

	_handleToggleSettings: function(event) {
		dojo.stopEvent(event);
		this.getSite().toggleSettings();
	},

	_updateTitle: function() {
		this.getSite().setTitle((this.getScopeItem() ? this.getScopeItem().name + " " : "") + "Event Log");
	}
});

})();
