/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.settings.AppearanceTab");

dojo.require("dojo.i18n");

dojo.require("com.ibm.team.dashboard.web.ui.internal.settings.AbstractTab");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {
	
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.settings.AppearanceTab",
	com.ibm.team.dashboard.web.ui.internal.settings.AbstractTab, {

	parseOnLoad: true,
	
	constructor: function(args) {
		this._messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		this.title = this._messages.settingsAppearance;
	},
	
	getNumberOfFields: function() {
		return this._settingsArea.viewletDefinition.dynamicTitle ? 4 : 3;
	},
	
	cancelSettings: function() {
		var workingCopy = this._settingsArea.workingCopy;
		var viewletContainer = this._settingsArea.viewletContainer;
		
		var trim = this._settingsArea.viewletHelper.getTrim();
		if (workingCopy.trim != undefined && trim != workingCopy.trim)
			viewletContainer.setTrim(trim);
			
		var showBgColor = this._settingsArea.viewletHelper.isShowBackground();
		if (workingCopy.showBackground != undefined && workingCopy.showBackground != showBgColor)
			viewletContainer.showBackground(showBgColor);
	},
	
	initWorkingCopy: function() {
		var workingCopy = this._settingsArea.workingCopy;
		var viewletHelper = this._settingsArea.viewletHelper;
		workingCopy.trim = viewletHelper.getTrim();
		workingCopy.title = viewletHelper.getTitle();
		workingCopy.userTitle = viewletHelper.isUserTitle();
		workingCopy.showBackground = viewletHelper.isShowBackground();
	},
	
	getDefaultValue: function(data) {
		if (data.id == "userTitle")
			return !this._settingsArea.workingCopy[data.id];
		return this._settingsArea.workingCopy[data.id];
	},
	
	setup: function() {
		this._settingsArea.tabContainer.addChild(this);
		this.startup();
		
		var title = this._settingsArea.viewletHelper.getTitle();
		
		var appearanceContent = new Array();
		var title = {name: this._messages.settingsTitle + "*", preferenceType: "STRING", id: "title"}		
		title.description = this._messages.settingsTitleDesc; 
		if (this._settingsArea.viewletDefinition.dynamicTitle) {
			appearanceContent.push({name: this._messages.settingsTitleComp, preferenceType: "BOOLEAN", id: "userTitle", description: this._messages.settingsTitleCompDesc});
			title.description = title.description + " " + this._messages.settingsTitleDesc2;
		}
		appearanceContent.push(title);
		if (this._settingsArea.viewletDefinition.dynamicTitle && !this._settingsArea.viewletHelper.isUserTitle())
			appearanceContent[1].startDisabled = true;
			
		appearanceContent.push({name: this._messages.settingsTrim, preferenceType: "COLOR", id: "trim", description: this._messages.settingsTrimDesc});
		
		var showBackground = this._settingsArea.viewletHelper.isShowBackground();
		appearanceContent.push({name: this._messages.settingsBackgr, preferenceType: "BOOLEAN", id:"showBackground", description: this._messages.settingsBackgrDesc});
		
		this.domNode.appendChild(this.createFormElements(appearanceContent));
		
		//this._fields["showBackgroundColor"];
	},
	
	saveSettings: function(newSettings) {
		var workingCopy = this._settingsArea.workingCopy;
		var viewletHelper = this._settingsArea.viewletHelper;
		var viewletContainer = this._settingsArea.viewletContainer;
		
		if (workingCopy.userTitle == false || workingCopy.userTitle == true) {
			//ignore the case where userTitle is undefined
			if (workingCopy.userTitle != this._settingsArea.oldSettings.userTitle)
				viewletHelper.setUserTitle(workingCopy.userTitle);
		}
		if ((!this._settingsArea.viewletDefinition.dynamicTitle || workingCopy.userTitle != false) && workingCopy.title && viewletContainer._headerPrimary.innerHTML != workingCopy.title) {
			viewletHelper.setTitle(workingCopy.title);
			viewletContainer.setTitle(workingCopy.title);
		}
		
		var trim = viewletHelper.getTrim();
		if (workingCopy.trim != undefined && workingCopy.trim != trim) {
			viewletHelper.setTrim(workingCopy.trim);
		}
		
		var showBgColor = viewletHelper.isShowBackground();
		if (workingCopy.showBackground != undefined && workingCopy.showBackground != showBgColor) {
			viewletHelper.setShowBackground(workingCopy.showBackground);
		}
	},
	
	createFormField: function(data, field, value){
		if (data.preferenceType == Constants.SettingsArea.FIELD_COLOR) {
			var inputDiv = document.createElement("div");
			dojo.addClass(inputDiv,"color-picker");
			
			var colorArr = ["blue","gray","gold","green","teal","purple","orange","no-trim"];
			for (var i = 0; i < colorArr.length; i++) {
				var div = document.createElement("div");
				div.className = colorArr[i]+"-option";
				inputDiv.appendChild(div);
				if (i == 0 || this._settingsArea.viewletHelper.getTrim() == i) {
					dojo.addClass(div,"selected");
					if (this.selectedColor) {
						dojo.removeClass(this.selectedColor,"selected");	
					}
					this.selectedColor = div;
				}
			}
			this.connect(inputDiv, "onclick", this._handleColorChanged);
			field.appendChild(inputDiv);
			return;
		}
		return this.inherited(arguments);
	},
	_handleCheckboxChange: function(event) {
		var box = event.target;
		if (box.name == "userTitle") {
			this._settingsArea.workingCopy.userTitle = !box.checked;
			if (this._fields["title"])
				this._fields["title"].disabled = !this._settingsArea.workingCopy.userTitle;
			this._settingsArea.settingChanged(box.name);	
		} else if (box.name == "showBackground") {
			this._settingsArea.workingCopy.showBackground = box.checked;
			this._settingsArea.settingChanged(box.name);
			this._settingsArea.viewletContainer.showBackground(box.checked);
		}
	},
	_handleTextboxChange: function(event) {
		var box = event.target;
		this._settingsArea.workingCopy.title = box.value;
		this._settingsArea.settingChanged(box.name);
	},
	_handleColorChanged: function(event){
		if (!event.target) return;
		var className = event.target.className;
		switch(className) {
			case "blue-option": 
				var trim = 0;
				break;
			case "gray-option":
				var trim = 1;
				break;
			case "gold-option":
				var trim = 2;
				break;
			case "green-option":
				var trim = 3;
				break;
			case "teal-option":
				var trim = 4;
				break;
			case "purple-option":
				var trim = 5;
				break;
			case "orange-option":
				var trim = 6;
				break;
			case "no-trim-option":
				var trim = 7;
				break;	
		}
		if (trim == undefined) return;
		dojo.addClass(event.target,"selected");
		if (this.selectedColor) {
			dojo.removeClass(this.selectedColor,"selected");	
		}
		this.selectedColor = event.target;
		this._settingsArea.workingCopy.trim = trim;
		this._settingsArea.viewletContainer.setTrim(trim);
		dojo.publish("com.ibm.team.dashboard.web.onresize");
	}
		
});

})();
