/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.settings.ScopeTab");

dojo.require("dojo.i18n");

dojo.require("com.ibm.team.dashboard.web.ui.internal.settings.AbstractTab");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.dashboard.web.client.internal.ItemNameRegistry");
dojo.require("com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {
	
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ItemNameRegistry = com.ibm.team.dashboard.web.client.internal.ItemNameRegistry;

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;

var SelectContributorDialog = com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.settings.ScopeTab",
	com.ibm.team.dashboard.web.ui.internal.settings.AbstractTab, {

	parseOnLoad: true,
	
	constructor: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		this.title = this._messages.settingsScope;
	},

	getNumberOfFields: function() {
		return this._settingsArea.viewletDefinition.scopeSubteams.indexOf("yes") == -1 ? 2 : 3;
	},
	
	isRequired: function() {
		return this._settingsArea.viewletDefinition.scopeSensitive ? true : false;
	},
	
	initWorkingCopy: function() {
		var workingCopy = this._settingsArea.workingCopy;
		var viewlet = this._settingsArea.viewlet;
		workingCopy.scope = viewlet.getScope();
		workingCopy.scopeItemId = viewlet.getScopeItem() ? viewlet.getScopeItem().itemId : null;
		if (this._settingsArea.viewletDefinition.scopeSubteams.indexOf("yes") != -1) {
			var value = this._settingsArea.viewletHelper.getScopeSubteams();
			if (!value && value != false)
				value = this._settingsArea.viewletDefinition.scopeSubteams == "yes-and-check";
			workingCopy.scopeSubteams = value;
		}
	},
	
	getDefaultValue: function(data) {
		switch(data.id) {
		case "scopeSubteams":
			var value = this._settingsArea.viewletHelper.getScopeSubteams();
			if (!value && value != false)
				value = this._settingsArea.viewletDefinition.scopeSubteams == "yes-and-check"
			return value;
		case "scopeItemContributor":
		case "scopeItemTeam":
		case "scopeItemProject":
			return this._settingsArea.viewletHelper.getScopeItem() ? this._settingsArea.viewletHelper.getScopeItem().itemId : null;
		case "scope":
			return this._settingsArea.viewletHelper.getScope();
		}
	},
	
	setup: function() {
		this._settingsArea.tabContainer.addChild(this);
		this.startup();
		var serviceObj = {
			self: this,
			success: function(/* SettingsAreaDTO */ settingsAreaDTO) {
				this.self._createContent(settingsAreaDTO);
			},
			failure: function(errorObj) {
				console.error(errorObj);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.getSettingsAreaInfo(getAuthenticatedContributor().userId, this._settingsArea.viewletHelper.getDashboardItem().getScope(),
			this._settingsArea.viewletHelper.getDashboardItem().getScopeItem().itemId, (this._settingsArea.viewlet.getScope() == DashboardConstants.SCOPE_CONTRIBUTOR && this._settingsArea.viewlet.getScopeItem() ? this._settingsArea.viewlet.getScopeItem().itemId : ""), srh);
	},
	
	saveSettings: function (newSettings) {
		// The working copy contains the evaluated scope (e.g. never "auto"), so we need to
		// compute the actual value here.
		if (this._fields["scope"]) {
			var value = this._fields["scope"].options[this._fields["scope"].selectedIndex].value;
			if (value) {
				value = parseInt(value);
				if (this._settingsArea.viewletHelper.getScope() != value)
					this._settingsArea.viewletHelper.setScope(value);
				if (value == -1)
					value = this._getApplicibleScope(this._settingsArea.viewletHelper.getDashboardItem().getScope());
				switch (value) {
					case DashboardConstants.SCOPE_CONTRIBUTOR:
						value = this._fields["scopeItemContributor"].options[this._fields["scopeItemContributor"].selectedIndex].value;
						if (value != -1)
							ItemNameRegistry.addName({itemId: value, name: this._fields["scopeItemContributor"].options[this._fields["scopeItemContributor"].selectedIndex].innerHTML});
						break;
					case DashboardConstants.SCOPE_TEAM_AREA:
						value = this._fields["scopeItemTeam"].options[this._fields["scopeItemTeam"].selectedIndex].value;
						if (value != -1)
							ItemNameRegistry.addName({itemId: value, name: this._fields["scopeItemTeam"].options[this._fields["scopeItemTeam"].selectedIndex].innerHTML});
						if (this._fields["scopeSubteams"])
							this._settingsArea.viewletHelper.setScopeSubteams(this._fields["scopeSubteams"].checked);
						break;
					case DashboardConstants.SCOPE_PROJECT_AREA:
						value = this._fields["scopeItemProject"].options[this._fields["scopeItemProject"].selectedIndex].value;
						if (value != -1)
							ItemNameRegistry.addName({itemId: value, name: this._fields["scopeItemProject"].options[this._fields["scopeItemProject"].selectedIndex].innerHTML});
						if (this._fields["scopeSubteams"]) {
							this._settingsArea.viewletHelper.setScopeSubteams(this._fields["scopeSubteams"].checked);
						}
						break;
				}
				if (!this._settingsArea.viewletHelper.getScopeItem() || this._settingsArea.viewletHelper.getScopeItem().itemId != value)
					this._settingsArea.viewletHelper.setScopeItem(value);
			}
		}
	},
	
	_createContent: function(/* SettingsAreaDTO */ settingsAreaDTO) {
		var comboData = this._buildScopeItemComboValues(settingsAreaDTO);
		
		var scopeContent = new Array();
		scopeContent.push({ name: this._messages.settingsScopeType, options: this._buildScopeComboValues(), preferenceType: "LIST", id: "scope", eventHandler: this._handleScopeTypeChanged, description: this._messages.settingsScopeTypeDesc });
		scopeContent.push({ name: this._messages.settingsScopeSel, options: comboData.contributors, preferenceType: "LIST", id: "scopeItemContributor", description: this._messages.settingsScopeSelContributorDesc });
		scopeContent.push({ name: this._messages.settingsScopeSel, groups: comboData.teams, preferenceType: "LIST", id: "scopeItemTeam", description: this._messages.settingsScopeSelTeamDesc });
		scopeContent.push({ name: this._messages.settingsScopeSel, options: comboData.projects, preferenceType: "LIST", id: "scopeItemProject", description: this._messages.settingsScopeSelProjectDesc });
		if (this._settingsArea.viewletDefinition.scopeSubteams == "yes" || this._settingsArea.viewletDefinition.scopeSubteams == "yes-and-check")
			scopeContent.push({ name: this._messages.settingsScopeSub, preferenceType: "BOOLEAN", id: "scopeSubteams", description: this._messages.settingsScopeSubDesc});
		var table = this.createFormElements(scopeContent);
		
		if (this._fields["scope"].options.length == 1)
			this._fields["scope"].disabled = true;
			
		
		this._setScopeItemMenu(this._settingsArea.viewletHelper.getScope());
		this._contributorComboSelectedIndex = this._fields["scopeItemContributor"].selectedIndex;
		
		this.domNode.appendChild(table);
	},
	
	_getApplicibleScope: function (scope) {
		switch (this._settingsArea.viewletDefinition.appliesTo) {
			case Constants.SettingsArea.SCOPE_APPLIES_TO_TEAMS:
				if (scope != DashboardConstants.SCOPE_TEAM_AREA) {
					scope = DashboardConstants.SCOPE_TEAM_AREA;
				}
				break;
			case Constants.SettingsArea.SCOPE_APPLIES_TO_CONTRIBUTORS:
				if (scope != DashboardConstants.SCOPE_CONTRIBUTOR) {
					scope = DashboardConstants.SCOPE_CONTRIBUTOR;
				}
				break;
			case Constants.SettingsArea.SCOPE_APPLIES_TO_PROJECTS:
				if (scope != DashboardConstants.SCOPE_PROJECT_AREA) {
					scope = DashboardConstants.SCOPE_PROJECT_AREA;
				}
				break;
			case Constants.SettingsArea.SCOPE_APPLIES_TO_PROCESS:
				if (scope == DashboardConstants.SCOPE_CONTRIBUTOR) {
					scope = DashboardConstants.SCOPE_PROJECT_AREA;
				}
				break;
		}
		return scope;
	},
	
	_setScopeItemMenu: function(scope, updateWorkingCopy) {
		var auto = false;
		if (scope != DashboardConstants.SCOPE_CONTRIBUTOR && scope != DashboardConstants.SCOPE_TEAM_AREA && scope != DashboardConstants.SCOPE_PROJECT_AREA) {
			scope = this._settingsArea.viewletHelper.getDashboardItem().getScope();
			auto = true;
		}
		var applicibleScope = this._getApplicibleScope(scope);
		auto = this._fields["scope"].options.length == 1 ? false : auto || scope != applicibleScope;
		var currentCombo;
		switch (applicibleScope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				currentCombo = this._fields["scopeItemContributor"];
				this._rows["scopeItemContributor"].style.display = "";
				this._rows["scopeItemTeam"].style.display = "none";
				this._rows["scopeItemProject"].style.display = "none";
				if (this._fields["scopeSubteams"])
					this._fields["scopeSubteams"].disabled = true;
				if (auto || this._settingsArea.viewletHelper.getDashboardItem().isPublic()) {
					this._fields["scopeItemContributor"].selectedIndex = 0;
					this._fields["scopeItemContributor"].disabled = true;
				} else
					this._fields["scopeItemContributor"].disabled = false;
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				currentCombo = this._fields["scopeItemTeam"];
				this._rows["scopeItemContributor"].style.display = "none";
				this._rows["scopeItemTeam"].style.display = "";
				this._rows["scopeItemProject"].style.display = "none";
				if (this._fields["scopeSubteams"])
					this._fields["scopeSubteams"].disabled = false;
				if (auto) {
					this._fields["scopeItemTeam"].selectedIndex = 0;
					this._fields["scopeItemTeam"].disabled = true;
				} else
					this._fields["scopeItemTeam"].disabled = false;
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				currentCombo = this._fields["scopeItemProject"];
				this._rows["scopeItemContributor"].style.display = "none";
				this._rows["scopeItemTeam"].style.display = "none";
				this._rows["scopeItemProject"].style.display = "";
				if (this._fields["scopeSubteams"])
					this._fields["scopeSubteams"].disabled = false;
				if (auto) {
					this._fields["scopeItemProject"].selectedIndex = 0;
					this._fields["scopeItemProject"].disabled = true;
				} else
					this._fields["scopeItemProject"].disabled = false;
				break;
		}
		if (updateWorkingCopy) {
			this._settingsArea.workingCopy.scope = applicibleScope;
			this._handleScopeItemChangedInternal(currentCombo.options[currentCombo.selectedIndex].value);
		}
	},
	
	_handleScopeTypeChanged: function(event) {
		this._setScopeItemMenu(parseInt(event.target.options[event.target.selectedIndex].value), /* updateWorkingCopy */ true);
		this._settingsArea.settingChanged("scope");
	},
	_handleComboboxChange: function(event) {
		var value = event.target.options[event.target.selectedIndex].value;
		if (this._fields["scope"] && this._fields["scope"].options[this._fields["scope"].selectedIndex].value == DashboardConstants.SCOPE_CONTRIBUTOR) {
			if (value == Constants.SettingsArea.CONTRIBUTOR_MORE_OPTION) {
				this._handleMoreContributors();
				return;
			}
			this._contributorComboSelectedIndex = event.target.selectedIndex;
		}
		this._handleScopeItemChangedInternal(value);
		this._settingsArea.settingChanged("scopeItem");
	},
	_handleCheckboxChange: function(event) {
		var box = event.target;
		this._settingsArea.workingCopy.scopeSubteams = box.checked;
		this._settingsArea.settingChanged("scopeSubteams");
	},
	_handleScopeItemChangedInternal: function(/* string */ value) {
		if (value == "-1") {
			var defaultObject = this._settingsArea.viewletHelper.getDashboardItem().getDefaultItem(this._settingsArea.workingCopy.scope || this._settingsArea.workingCopy.scope == 0 ? this._settingsArea.workingCopy.scope : this._settingsArea.viewlet.getScope());
			this._settingsArea.workingCopy.scopeItemId = defaultObject == null ? null : defaultObject.itemId;
		}
		else
			this._settingsArea.workingCopy.scopeItemId = value;
	},
	_handleMoreContributors: function() {
		var self= this;
		this._fields["scopeItemContributor"].selectedIndex = this._contributorComboSelectedIndex;
		var params= {
			title: this._messages.settingsScopeSelContributor,
			isMultiselect: false,
			useOkCancel: true, /* Not Add/Close */
			onOK: function(selectedContribs) {
				if (selectedContribs && selectedContribs.length > 0) {
					var selectionPresent = false;
					var insertIndex = -1;
					// index 1 to length -1 ignores "Auto" and "More"
					for (var i = 1; i < self._fields["scopeItemContributor"].options.length - 1; i++) {
						if (self._fields["scopeItemContributor"].options[i] && self._fields["scopeItemContributor"].options[i].value == selectedContribs[0].itemId) {
							self._fields["scopeItemContributor"].selectedIndex = i;
							selectionPresent = true;
							break;
						} else if (self._fields["scopeItemContributor"].options[i] && self._fields["scopeItemContributor"].options[i].innerHTML > selectedContribs[0].name) {
							insertIndex = i;
							break;
						}
					}
					// if the option wasn't in the list, insert it
					if (!selectionPresent) {
						var listAtMaxLength = self._fields["scopeItemContributor"].options.length > 11;
						// if we didn't find an index to insert at, it comes alphabetically last
						// if we are going to remove an item from the end, put the new option second last
						if (insertIndex == -1)
							insertIndex = self._fields["scopeItemContributor"].options.length - (listAtMaxLength ? 2 : 1);
						var option = document.createElement("option");
						option.innerHTML = selectedContribs[0].name;
						option.value = selectedContribs[0].itemId;
						// insert and select the new option
						self._fields["scopeItemContributor"].insertBefore(option,self._fields["scopeItemContributor"].options[insertIndex]);
						self._fields["scopeItemContributor"].selectedIndex = insertIndex;
						// if the list was at max length, remove the last item
						if (listAtMaxLength)
							self._fields["scopeItemContributor"].removeChild(self._fields["scopeItemContributor"].options[self._fields["scopeItemContributor"].options.length-2]);
						self._handleScopeItemChangedInternal(selectedContribs[0].itemId);
					}
					self._contributorComboSelectedIndex = self._fields["scopeItemContributor"].selectedIndex;
				}
			}
		};
		new SelectContributorDialog(params);
	},
	
	_buildScopeComboValues: function() {
		var auto = this._settingsArea.viewletHelper.getDashboardItem().getScope();
		var autoValue;
		var result = dojo.clone(Constants.SettingsArea.Scopes);
		switch(this._settingsArea.viewletDefinition.appliesTo) {
			case Constants.SettingsArea.SCOPE_APPLIES_TO_TEAMS:
				delete result.PROJECT_AREA;
				delete result.CONTRIBUTOR;
				auto = DashboardConstants.SCOPE_TEAM_AREA;
				break;
			case Constants.SettingsArea.SCOPE_APPLIES_TO_PROJECTS:
				delete result.TEAM_AREA;
				delete result.CONTRIBUTOR;
				auto = DashboardConstants.SCOPE_PROJECT_AREA;
				break;
			case Constants.SettingsArea.SCOPE_APPLIES_TO_PROCESS:
				delete result.CONTRIBUTOR;
				if (auto == DashboardConstants.SCOPE_CONTRIBUTOR)
					auto = DashboardConstants.SCOPE_PROJECT_AREA;
				break;
			case Constants.SettingsArea.SCOPE_APPLIES_TO_CONTRIBUTORS:
				delete result.TEAM_AREA;
				delete result.PROJECT_AREA;
				delete result.DEFAULT;
				auto = DashboardConstants.SCOPE_CONTRIBUTOR;
				break;
		}
		if (result.DEFAULT) {
			switch(auto) {
				case DashboardConstants.SCOPE_CONTRIBUTOR:
					autoValue = result.CONTRIBUTOR.label;
					break;
				case DashboardConstants.SCOPE_TEAM_AREA:
					autoValue = result.TEAM_AREA.label;
					break;
				case DashboardConstants.SCOPE_PROJECT_AREA:
					autoValue = result.PROJECT_AREA.label;
					break;
			}
			this._autoScope = auto;
			result.DEFAULT.label = result.DEFAULT.label.replace("{1}", autoValue);
		}
		return result;
	},
	_buildScopeItemComboValues: function(/* SettingsAreaDTO */ dto) {
		var projects = new Array();
		var teams = new Array();

		var defaultTeam = dojo.clone(Constants.SettingsArea.Scopes.DEFAULT);
		var defaultTeamObject = this._settingsArea.viewletHelper.getDashboardItem().getDefaultItem(DashboardConstants.SCOPE_TEAM_AREA);
		defaultTeam.label = defaultTeam.label.replace("{1}", defaultTeamObject == null ? this._messages.settingsScopeNone : defaultTeamObject.name);
		teams.push({options: [defaultTeam]});

		var defaultProject = dojo.clone(Constants.SettingsArea.Scopes.DEFAULT);
		defaultProject.label = defaultProject.label.replace("{1}", this._settingsArea.viewletHelper.getDashboardItem().getDefaultItem(DashboardConstants.SCOPE_PROJECT_AREA).name);
		projects.push(defaultProject);

		for (var x in dto.projectAreas) {
			projects.push({value: dto.projectAreas[x].itemId, label: dto.projectAreas[x].name});
			var options = this._populateTeams(dto.projectAreas[x].children, new Array(), 10);
			teams.push({name: dto.projectAreas[x].name, options: options});
		}
		var contributors = new Array();
		var defaultContributor = dojo.clone(Constants.SettingsArea.Scopes.DEFAULT);
		var defaultContributorObject = this._settingsArea.viewletHelper.getDashboardItem().getDefaultItem(DashboardConstants.SCOPE_CONTRIBUTOR);
		defaultContributor.label = defaultContributor.label.replace("{1}", defaultContributorObject == null? this._messages.settingsScopeNone : defaultContributorObject.name);
		contributors.push(defaultContributor);
		for (var x in dto.contributors)
			contributors.push({value: dto.contributors[x].itemId, label: dto.contributors[x].name});
		contributors.push({value: Constants.SettingsArea.CONTRIBUTOR_MORE_OPTION, label: this._messages.settingsScopeMore});
		return {projects: projects, teams: teams, contributors: contributors};
	},
	_populateTeams: function(/* ProcessAreaDTO[] */ dtos, /* Array */ options, /* number */ indent) {
		for (var x in dtos) {
			options.push({value: dtos[x].itemId, label: dtos[x].name, padding: indent});
			options = this._populateTeams(dtos[x].children, options, indent + 10);
		}
		return options;
	}
		
});

})();
