/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.AbstractGroupMode"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.common.TreeNode"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.SortModes"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.BarModes"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.GroupWidget"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.WorkItemWidget"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.common.ProgressInformation"); //$NON-NLS-1$


(function() {

var Assert				= com.ibm.team.apt.web.ui.internal.common.Assert;
var TreeNode			= com.ibm.team.apt.web.ui.internal.view.common.TreeNode;
var GroupWidget			= com.ibm.team.apt.web.ui.internal.view.plan.planneditems.GroupWidget;
var WorkItemWidget		= com.ibm.team.apt.web.ui.internal.view.plan.planneditems.WorkItemWidget;
var SortModes			= com.ibm.team.apt.web.ui.internal.view.plan.planneditems.SortModes;
var BarModes			= com.ibm.team.apt.web.ui.internal.view.plan.planneditems.BarModes;
var ProgressInformation	= com.ibm.team.apt.web.ui.internal.common.ProgressInformation;

var NT_ROOT				= "root"; //$NON-NLS-1$
var NT_GROUP			= "group"; //$NON-NLS-1$
var NT_WORKITEM			= "workItem"; //$NON-NLS-1$

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.AbstractGroupMode", null, { //$NON-NLS-1$
	
	constructor: function(plannedItemsPage) {
		this._plannedItemsPage= plannedItemsPage;
		this._rootNode= null;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	getGroupModeId: function() {
		Assert.isTrue(false, "getGroupModeId must be implemented"); //$NON-NLS-1$
	},

	getGroupModeLabel: function() {
		Assert.isTrue(false, "getGroupModeLabel must be implemented"); //$NON-NLS-1$
	},

	getSortModes: function() {
		return [SortModes.priority, SortModes.estimate];
	},

	getBarModes: function() {
		return [BarModes.progress];
	},

	getGroupSorter: function() {
		return null;
	},
	
	getColumns: function() {
		Assert.isTrue(false, "getColumns must be implemented"); //$NON-NLS-1$
	},

	buildNodeTree: function(workItems) {
		this._rootNode= new TreeNode("", NT_ROOT, null, null, null); //$NON-NLS-1$
		
		dojo.forEach(this.getDefaultGroupIds(), dojo.hitch(this, function(groupId){
			this._getOrCreateGroupNode(groupId);
		}));
		
		for (var i= 0; i < workItems.length; i++) {
			this._processWorkItem([workItems[i]]);
		}
		
		return this._rootNode;
	},
	
	// ---- spi ------------------------------------------------------------------------------------------------------------

	getGroupId: function(workItemStack) {
		return ""; //$NON-NLS-1$
	},

	getDefaultGroups: function() {
		return [];
	},

	getGroupDescription: function(groupId) {
		return {
			path: [],
			name: "", //$NON-NLS-1$
			image: "" //$NON-NLS-1$
		};
	},

	geWorkItemWidgetOptions: function(workItem) {
		return null;
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------

	_processWorkItem: function(workItemStack) {
		var allGroupsIds= {};
		for (var i= 0; i < workItemStack.length; i++) {
			var groupId= this.getGroupId(workItemStack.slice(0, i + 1));
			if (groupId != null) {
				allGroupsIds[groupId]= i == (workItemStack.length -1);
			}
		}

		for (var groupId in allGroupsIds) {
			var node= this._getOrCreateGroupNode(groupId);
			for (var i = 0; i < workItemStack.length; i++) {
				node= this._getOrCreateWorkItemNode(workItemStack[i], node);
			}
			
			if (allGroupsIds[groupId]) {
				node.setTag("primary"); //$NON-NLS-1$
			}			
		}

		var workItem= workItemStack[workItemStack.length - 1];
		if (dojo.isArray(workItem.children)) {
			for (var i= 0; i < workItem.children.length; i++) {
				workItemStack.push(workItem.children[i]);		
				try {
					this._processWorkItem(workItemStack);
				} finally {
					workItemStack.pop();
				}
			}
		}
	},
	
	_getOrCreateGroupNode: function(groupId) {
		if (groupId === "") //$NON-NLS-1$
			return parentNode;
			
		var groupDescription= this.getGroupDescription(groupId);
		var parentNode= this._rootNode;
		
		if (dojo.isArray(groupDescription.path) && groupDescription.path.length > 0) {
			for (var i= 0; i < groupDescription.path.length; i++) {
				parentNode= this._getOrCreateGroupNode(groupDescription.path[i]);
			}
		}
		
		var result= parentNode.getChild(groupId);
		if (result == null) {
			result = new TreeNode(groupId, NT_GROUP, groupDescription, dojo.hitch(this, this._groupWidgetCreatorFn), parentNode);
		}
		return result;
	},

	_getOrCreateWorkItemNode: function(workItem, parentNode) {
		var result= parentNode.getChild(workItem.itemId);
		if (result == null) {
			result = new TreeNode(workItem.itemId, NT_WORKITEM, workItem, dojo.hitch(this, this._workItemWidgetCreatorFn), parentNode);
		}
		return result;
	},

	_workItemWidgetCreatorFn: function(treeNode, parentTreeItem) {
		var workItem= treeNode.getNodeValue();
		
		var options= this.geWorkItemWidgetOptions(workItem) || {};
		options["isPlaceholder"]= !treeNode.hasTag("primary") || treeNode.hasTag("unfilteredChildren"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		
		return new WorkItemWidget(WorkItemWidget.constructorArgs(workItem, options, parentTreeItem, this._plannedItemsPage));
	},
	
	_groupWidgetCreatorFn: function(treeNode, parentTreeItem) {
		var groupDescription= treeNode.getNodeValue();
		var progresInfo= this._calculateProgress(treeNode);
		return new GroupWidget(GroupWidget.constructorArgs(groupDescription, progresInfo, parentTreeItem, this._plannedItemsPage));
	},

	_getReferencedItem: function(type, id) {
		return this._plannedItemsPage._getItem(type, id);
	},
	
	_calculateProgress: function(treeNode) {
		var result= new ProgressInformation();
		
		treeNode.accept(function(node){
			if (node.getNodeType() == NT_WORKITEM) {
				var planItem= node.getNodeValue();
				if (node.hasTag("primary") || planItem.isProgressPlanItem()) { //$NON-NLS-1$
					planItem.updateProgressInformation(result);
				}
			}
			return true;
		});

		return result;
	},

	__sentinel: null // terminates this class definition
});

})();
