/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.widgets.ContributorWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function(){

var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var ServiceResponseHandler= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemClient= com.ibm.team.workitem.web.client.internal.WorkItemClient;
var SelectContributorDialog = com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog;

var Messages= dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.widgets.ContributorWidget", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/widgets/templates/ContributorWidget.html"),
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {
		this.presentation= args.presentation;
		this.selection= args.selection;
		this.errorCallback= args.errorCallback;
		this._hasSelection= false;
	},
	
	postCreate: function() {
		
		this._select.size= 6;
		this._select.multiple= "multiple";

		this._createAddContributorButton();
		var attributeId= this.presentation.attribute.id;
		var srh= new ServiceResponseHandler(this, "_initValues", "_handleError");
		WorkItemClient.getTeamMembers(srh);
	},
	
	destroy: function() {
		try {
			if(this.childButton) this.childButton.destroy();
			this.inherited(arguments);
		} catch (e) {
			
		}
	},
	
	/* String[] */ getValues: function() {
		
		if (!this._hasSelection)
			return this._getInitialSelectionValues();
			
		var selection= new Array();
		var select= this._select;
		for (var i= 0; i < select.length; i++) {
			if (select[i].selected) {
				selection.push(select[i].value);
			}
		}
		return selection;
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Internals

	
	_getInitialSelectionValues: function() {
		var values= this.selection || [];
		var selection= new Array();
		for (var i= 0; i < values.length; i++) {
			selection.push(values[i].id);
		}
		return selection;
	},
		
	_onSelectionChange: function() {
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
	_createAddContributorButton: function() {
		var self= this;
		var addUserButtonParameters= {
			label: Messages.labelAddUser,
			onClick: function() {
				self._handleAddUser();
			}
		};
		var addUserButton= new dijit.form.Button(addUserButtonParameters);
		this._addUserButton.appendChild(addUserButton.domNode);
		
		this.childButton = addUserButton; 
	},
	
	_handleAddUser: function() {
		var self= this;
		var params= {
				title: Messages.dialogTitleAddUsers,
				parentElement: this.domNode,
				disableUsers: [],
				onOK: function(selectedContribs) {
					self._addUsers(selectedContribs);
				}
			};
		new SelectContributorDialog(params);
			
	},
	
	_addUsers: function(selected) {
		for (var i= 0; i < selected.length; i++) {
			var user= selected[i];
			var userOption= this._findUserOption(user);
			if (userOption) {
				userOption.selected= true;
			} else {
				this._addValues([ user ], true)
			}
		}
		this._hasSelection= true;
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
	_findUserOption: function(user) {
		var select= this._select;
		for (var i= 0; i < select.length; i++) {
			if (select[i].value == user.itemId) {
				return select[i];
			}
		}
		return null;
	},
	
	_initValues: function(/* ContributorDTO[] */ valueSet) {
		if (!valueSet || valueSet.length == 0)
			return;
		this._select.length= 0;
		this._addValues(valueSet);
		this._setSelection();
	},
	
	_handleError: function(e) {
		if (this.errorCallback)
			this.errorCallback(Messages.errorLoadingUsersFailed, e);
	},
	
	_addValues: function(/* ContributorDTO[] */ values, /* boolean */ selectValue) {
		if (values) {
			var index= this._select.length;
			for (var i= 0; i < values.length; i++) {
				var option= this._createOption(values[i]);
				if (selectValue)
					option.selected= true;
				this._select[index++]= option;
			}
		}
	},
	
	_createOption: function(/* ContributorDTO */ item) {
		var option= new Option(item.name, item.itemId, false, false);
		return option;
	},
	
	_setSelection: function() {
		this._hasSelection= true;
		if (this.selection) {
			for (var i= 0; i < this.selection.length; i++) {
				var selected= this.selection[i];
				var option= this._findUserOption({ itemId: selected.id });
				if (option) {
					option.selected= true;
				} else {
					this._addValues( [{name: selected.label, itemId: selected.id }], true);
				}
			}
		}
	}
	
});
	
})();
